
#ifndef LABPLOT_EXPORT_H
#define LABPLOT_EXPORT_H

#ifdef LABPLOT_STATIC_DEFINE
#  define LABPLOT_EXPORT
#  define LABPLOT_NO_EXPORT
#else
#  ifndef LABPLOT_EXPORT
#    ifdef liblabplot_EXPORTS
        /* We are building this library */
#      define LABPLOT_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define LABPLOT_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef LABPLOT_NO_EXPORT
#    define LABPLOT_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef LABPLOT_DEPRECATED
#  define LABPLOT_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef LABPLOT_DEPRECATED_EXPORT
#  define LABPLOT_DEPRECATED_EXPORT LABPLOT_EXPORT LABPLOT_DEPRECATED
#endif

#ifndef LABPLOT_DEPRECATED_NO_EXPORT
#  define LABPLOT_DEPRECATED_NO_EXPORT LABPLOT_NO_EXPORT LABPLOT_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef LABPLOT_NO_DEPRECATED
#    define LABPLOT_NO_DEPRECATED
#  endif
#endif

#endif /* LABPLOT_EXPORT_H */
